import cv2 as cv

class BaseTracker:
    """Base class for all trackers."""

    def __init__(self, params):
        self.params = params

    def initialize(self, raw_im, image, state):
        """Overload this function in your tracker. This should initialize the model."""
        raise NotImplementedError

    def track(self, raw_im, image):
        """Overload this function in your tracker. This should track in the frame and update the model."""
        raise NotImplementedError

    def _read_image(self, image_file: str):
        raw_im = cv.imread(image_file)
        im = cv.cvtColor(raw_im.copy(), cv.COLOR_BGR2RGB)
        return raw_im, im

