import importlib
import os
import pickle
from libs.utils.settings import local_env_settings


class Tracker:
    """Wraps the tracker for evaluation and running purposes.
    args:
        name: Name of tracking method.
        parameter_name: Name of parameter file.
        run_id: The run id.
    """

    def __init__(self, name: str, parameter_name: str):
        self.name = name
        self.parameter_name = parameter_name

        tracker_module = importlib.import_module('libs.tracker.{}'.format(self.name))

        self.parameters = self.get_parameters()
        self.tracker_class = tracker_module.get_tracker_class()

    def get_parameters(self):
        """Get parameters."""

        param_module = importlib.import_module('star_tracking.parameter.{}.{}'.format(self.name, self.parameter_name))
        params = param_module.parameters()

        return params


