#!/usr/bin/python
import os.path as osp
import sys

def add_path(path):
    if path not in sys.path:
        sys.path.insert(0, path)
this_dir = osp.dirname(__file__)
lib_path = osp.join(this_dir, '..')
add_path(lib_path)

import numpy as np

from libs.tracker.tracker import Tracker


class STARTracker(object):
    def __init__(self, raw_im, image, region):
        # init tracker
        self.tracker = Tracker('fstar', 'res50_512_TLVC')
        self.tracker = self.tracker.tracker_class(self.tracker.parameters)
        cx, cy, w, h = self.get_axis_aligned_bbox(region)
        init_state = [cx, cy, w, h]
        self.tracker.initialize(raw_im, image, init_state)

    def get_axis_aligned_bbox(self, region):
        nv = region.size
        if nv == 8:
            cx = np.mean(region[0::2])
            cy = np.mean(region[1::2])
            x1 = min(region[0::2])
            x2 = max(region[0::2])
            y1 = min(region[1::2])
            y2 = max(region[1::2])
            A1 = np.linalg.norm(region[0:2] - region[2:4]) * np.linalg.norm(region[2:4] - region[4:6])
            A2 = (x2 - x1) * (y2 - y1)
            s = np.sqrt(A1 / A2)
            w = s * (x2 - x1) + 1
            h = s * (y2 - y1) + 1
        else:
            x = region[0]
            y = region[1]
            w = region[2]
            h = region[3]
            cx = x + w / 2
            cy = y + h / 2

        return cx, cy, w, h

    def track(self, raw_im, image):

        state, score = self.tracker.track(raw_im, image)

        if score > 1:
            score = float(1)
        elif score < 0:
            score = float(0)
        else:
            score = float(score)

        return state, score

def main():
    import cv2
    im = cv2.imread("/data/home/v-had/debug.jpg")
    tracker = STARTracker(im, im, np.array([10.,20.,100.,200.]))
    cnt = 0
    while cnt < 100:
        im = cv2.imread("/data/home/v-had/debug.jpg")
        state, core = tracker.track(im, im)
        print(core)


if __name__ == '__main__':
    main()

